/*
 * nb/as.c - peform an "adapter status" on the given NetBios server, this
 *	lists the names that host has registered in the distributed namespace.
 *	we also have an educated guess at what program registered each name.
 */
#include <u.h>
#include <libc.h>
#include <ip.h>
#include <ctype.h>
#include <bio.h>
#include "netbios.h"

char *unique[] = {
	[0x00] 	"MS workstation service",
	[0x01]	"MS messenger service",
	[0x03]	"MS WinPopup",
	[0x06]	"RAS server",
	[0x1b]	"Domain Master Browser",
	[0x1d]	"Master Browser",
	[0x1F]	"NetDDE",
	[0x20]	"MS File sharing",
	[0x21]	"RAS client",
	[0x22]	"MS Exchange Interchange",
 	[0x23]	"MS Exchange Store",
	[0x24]	"MS Exchange Directory",
	[0x2B]	"Lotus Notes Server",
	[0x30]	"MS Modem Derver",
	[0x31]	"MS Modem Client",
	[0x42]	"McAfee anti-virus",
	[0x43]	"SMS control client",
	[0x44]	"SMS control admin",
	[0x45]	"SMS chat client",
	[0x46]	"SMS file client",
	[0x4C]	"DEC Pathworks",
	[0x52]	"DEC Pathworks",
	[0x6A]	"MS Exchange IMC",
	[0x87]	"MS Exchange MTA",
	[0xBE]	"MS NetMon Agent",
	[0xBf]	"MS NetMon Application",
};

char *group[] = {
	[0x00]	"Domain name",
	[0x01]	"Local Master Browser",
	[0x03]	"Messenger Service",
	[0x1B]	"Domain Master Browser",
	[0x1C]	"Internet Domain Controller",
	[0x1D]	"LAN Local Master Browser",
	[0x1E]	"Browser Election Service",
	[0x20]	"DCA IrmaLan Gateway Server",
	[0x2F]	"Lotus Notes",
	[0x33]	"Lotus Notes",
};

void
nicvendor(uchar mac[6])
{
	Biobuf *bp;
	char *line;
	int len, l, best;
	char local[16], match[128];

	if((bp = Bopen("/sys/lib/ether-vendors", OREAD)) == nil)
		return;

	sprint(local, "%02x%02x%02x%02x%02x%02x",
		mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);

	best = 0;
	len = strlen(local);
	while((line = Brdline(bp, '\n')) != nil){
		line[Blinelen(bp)-1] = 0;
		for(l = 4; l < len; l++){
			if(strncmp(local, line, l) == 0){
				if(l > best){
					snprint(match, sizeof(match), line+7);
					best = l;
				}
			}
		}
	}
	print("NIC vendor           %s\n", match);
	Bterm(bp);
}

static void
ding(void *u, char *msg)
{
	USED(u);
	if(strstr(msg, "alarm"))
		noted(NCONT);
	noted(NDFLT);
}

static void
usage(void)
{
	fprint(2, "usage: %s [-x netmtpt] sysname\n", argv0);
	exits("usage");
}

void
main(int argc, char **argv)
{
	uchar *p, mac[6], buf[1024];
	char *s, net[64], name[Rnamlen];
	int err, rtrn, ltrn, flg, svs, i, n, fd;

	setnetmtpt(net, sizeof(net), nil);
	ARGBEGIN{
	case 'x':
		setnetmtpt(net, sizeof(net), EARGF(usage()));
		break;
	}ARGEND;

	if (argc != 1)
		usage();

	if((fd = dial(netmkaddr(argv[0], "udp", "137"), "137", 0, 0)) < 0)
		sysfatal("can't dial - %s : %r", argv[0]);

	ltrn = getpid() & 0xffff;

	p = buf;
	p16(&p, ltrn);			// ID
	p8(&p, 0);			// flags (unicast)
	p8(&p, OPtQuery);		// type (query)
	p16(&p, 1);			// # questions
	p16(&p, 0);			// # answers
	p16(&p, 0);			// # authritative answers
	p16(&p, 0);			// # additional answers
        pnbname(&p, "*", '\0', '\0');	// question name
	p16(&p, RRstat);		// question type (node status)
	p16(&p, Cinet);			// question class (inet)

	if (write(fd, buf, p-buf) != p-buf)
		sysfatal("can't write: %r");

	notify(ding);

	n = 0;
	memset(buf, 0, sizeof(buf));
	for (i = 0; i < 3; i++){
		alarm(1000);
		n = read(fd, buf, sizeof(buf));
		alarm(0);

		p = buf;
		rtrn = g16(&p);
		if (n != -1 && rtrn == ltrn)
			break;
	}
	close(fd);

	if(n == -1)
		sysfatal("%s timeout\n", argv[0]);
	g16(&p);			// flags
	err = g16(&p);			// err
	g16(&p);			// type
	g16(&p);			// # questions
	g16(&p);			// # answers
	gnbname(&p, name, Rnamlen); 	// name in request
	g16(&p);			// type in request
	g16(&p);			// class in request
	g32(&p);			// TTL (always zero)
	g16(&p);			// reply data len
	n = g8(&p);			// number of names

	if(((err >> 11) & 7) != 0)
		sysfatal("%s\n", nberr((err >> 11) & 7));

	for (i = 0; i < n; i++){
		gmem(&p, name, 16);
		svs = name[15];
		name[15] = 0;
		flg = g16(&p);
		s = (flg & 0x8000)? group[svs]: unique[svs];
		if (s)
			print("%-20s %s\n", name, s);
		else
			print("%-20s id=0x%02ux type=%s\n", name, svs,
				(flg & 0x8000)? "group": "unique");
	}

	for(i = 0; i < 6; i++)
		mac[i] = g8(&p);
	print("%02x%02x%02x%02x%02x%02x         MAC address\n",
		mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	nicvendor(mac);

	/* Sadly the remainder of the stats seem to always be zero */
}

