/*
 * nbdc.c - get the IP address of the Domain controller
 *
 */
#include <u.h>
#include <libc.h>
#include <ip.h>
#include <ctype.h>
#include "netbios.h"

static void
ding(void *u, char *msg)
{
	USED(u);

	if(strstr(msg, "alarm"))
		noted(NCONT);
	noted(NDFLT);
}

int
mybcast(uchar *ip, char *net)
{
	int i;
	Ipifc *nifc;
	Iplifc *lifc;
	static Ipifc *ifc;
	uchar mynet[IPaddrlen];
	static uchar loopbacknet[IPaddrlen] = {
		0, 0, 0, 0,
		0, 0, 0, 0,
		0, 0, 0xff, 0xff,
		127, 0, 0, 0
	};
	static uchar loopbackmask[IPaddrlen] = {
		0xff, 0xff, 0xff, 0xff,
		0xff, 0xff, 0xff, 0xff,
		0xff, 0xff, 0xff, 0xff,
		0xff, 0, 0, 0
	};
	
	
	ifc = readipifc(net, ifc, -1);
	for(nifc = ifc; nifc; nifc = nifc->next)
		for(lifc = nifc->lifc; lifc; lifc = lifc->next){
			maskip(lifc->ip, loopbackmask, mynet);
			if(ipcmp(mynet, loopbacknet) == 0){
				continue;
			}
			if(ipcmp(lifc->ip, IPnoaddr) != 0){
				for (i = 0; i < IPaddrlen; i++)
					ip[i] = lifc->net[i] | ~lifc->mask[i];
				return 0;
			}
		}
	ipmove(ip, IPnoaddr);
	return -1;
}

static void
usage(void)
{
	fprint(2, "usage: %s [-x netmtpt] domain\n", argv0);
	exits("usage");
}


void
main(int argc, char **argv)
{
	Udphdr *up;
	uchar *p, buf[1024];
	int ttl, flags, tid_rx, tid_tx, cfd, i, n, fd;
	char name[Rnamlen], net[32], data[128], devdir[40];

	setnetmtpt(net, sizeof(net), nil);
	ARGBEGIN{
	case 'x':
		setnetmtpt(net, sizeof(net), EARGF(usage()));
		break;
	}ARGEND;

	if(argc != 1)
		usage();

	snprint(data, sizeof(data), "%s/udp!*!137", net);
	if((cfd = announce(data, devdir)) == -1)
		sysfatal("can't announce: %r");
	if(fprint(cfd, "headers") < 0)
		sysfatal("can't set header mode: %r");
	snprint(data, sizeof(data), "%s/data", devdir);
	if((fd = open(data, ORDWR)) == -1)
		sysfatal("can't open udp data");

	tid_tx = (getpid() ^ time(nil)) & 0xffff;
	fmtinstall('I', eipfmt);

	up = (Udphdr *)buf;
	mybcast(up->raddr, net);
	hnputs(up->rport, 137);

	p = buf+Udphdrsize;
	p16(&p, tid_tx);		// Transaction ID
	p16(&p, 0x0110);		// flags: broadcast & recusion desired
	p16(&p, 1);			// # questions
	p16(&p, 0);			// # answers
	p16(&p, 0);			// # authorities
	p16(&p, 0);			// # additional
        pnbname(&p, argv[0], Tdomain, ' '); // question name
	p16(&p, 0x20);			// question type (name query)
	p16(&p, 1);			// question class (internet)

	if(write(fd, buf, p-buf) != p-buf)
		sysfatal("can't write: %r");

	notify(ding);

	memset(buf, 0, sizeof(buf));
	for (i = 0; i < 4; i++){
		alarm(5000);
		n = read(fd, buf, sizeof(buf));
		alarm(0);
		if(n == -1 || n < 4)
			continue;

		p = buf +Udphdrsize;
		tid_rx = g16(&p);			// Transaction ID
		flags = g16(&p);		// flags
		if(tid_rx == tid_tx && (flags & Freply) != 0)
			break;
	}
	close(fd);
	close(cfd);

	if(i >= 4)
		exits("timeout");

	g16(&p);			// # questions
	g16(&p);			// # answers
	g16(&p);			// # authorities
	g16(&p);			// # aditional
	gnbname(&p, name, Rnamlen); 	// name in request
	g16(&p);			// type in request
	g16(&p);			// class in request
	ttl = g32(&p);			// TTL
	n = g16(&p);			// reply data len

	for(i = 0; i < n; i += 6){
		int flgs = g16(&p);
		int a = g8(&p);
		int b = g8(&p);
		int c = g8(&p);
		int d = g8(&p);
		USED(flgs);
		print("windc=%s ip=%d.%d.%d.%d ttl=%d\n",
			name, a, b, c, d, ttl);
	}
}

