/* RFC2136 DNS inform - necessary for Win2k3 DNS servers */
#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ndb.h>
#include <ip.h>
#include "netbios.h"

enum {
	FQDNMAX	= 255,

	Cin	= 1,		// internet
	Call	= 255,		// all classes

	Ta	= 1,		// address
	Taaaa	= 28,
	Tsoa	= 6,		// start of authority
	Tall	= 255,		// all types

};

char *dnsrch[] = {
	"dnsdomain",
	"dom"
};

void
usage(void)
{
	fprint(2, "usage: %s [-x netmtpt]\n", argv0);
	exits("usage");
}

void
ding(void *u, char *msg)
{
	USED(u);

	if(strstr(msg, "alarm"))
		noted(NCONT);
	noted(NDFLT);
}

void
enc_name(uchar **p, char *s)
{
	uchar *len;

	while (*s){
		len = (*p)++;
		while(*s && *s != '.')
			*(*p)++ = *s++;
		*len = (*p - len) -1;
		if(*s == '.')
			s++;
	}
	*(*p)++ = 0;
}

void
main(int argc, char *argv[])
{
	Ndb *db;
	Ndbtuple *t, *tt;
	ushort rxid, txid;
	int debug, len, err, fd;
	char *sysname, *dnsdomain, *dom, *ns, *s, net[32];
	uchar *p, buf[4096], addr[IPv4addrlen], v6addr[IPaddrlen];
	static char *query[] = { "dom", "dnsdomain", "ns" };

	fmtinstall('I', eipfmt);
	fmtinstall('V', eipfmt);
	setnetmtpt(net, sizeof(net), nil);

	debug = 0;
	ns = nil;
	dom = nil;
	dnsdomain = nil;
	ARGBEGIN{
	case 'd':
		debug = 1;
		break;
	case 'x':
		;
		if((s = ARGF()) == nil)
			usage();
		setnetmtpt(net, sizeof(net), s);
		break;
	}ARGEND;

	if(argc != 0)
		usage();

	if((sysname = getenv("sysname")) == nil)
		sysfatal("$sysname not set");
 
	if((db = ndbopen(nil)) == nil)
		sysfatal("open ndb - %r\n");
	tt = ndbipinfo(db, "sys", sysname, query, nelem(query));
	for(t = tt; t; t = t->entry){
		if (strcmp(t->attr, "ns") == 0)
			ns = t->val;
		if (strcmp(t->attr, "dom") == 0)
			dom = t->val;
		if (strcmp(t->attr, "dnsdomain") == 0)
			dnsdomain = t->val;
	}
	ndbfree(tt);
	ndbclose(db);

	if(! ns)
		sysfatal("no relevant ns=");
	if(! dom)
		sysfatal("no relevant dom=");
	if(! dnsdomain)
		sysfatal("no relevant dnsdomain=");

	myipaddr(v6addr, net);
	memcpy(addr, v6addr+(IPaddrlen-IPv4addrlen), IPv4addrlen);

	if(debug){
		print("ip=%V\n", addr);
		print("ns=%s\n", ns);
		print("dnsdomain=%s\n", dnsdomain);
		print("dom=%s\n", dom);
	}

	if((fd = dial(netmkaddr(ns, "udp", "dns"), 0, 0, 0)) < 0)
		sysfatal("%s can't dial - %r", ns);

	txid = time(nil) + getpid();

	p = buf;
	p16(&p, txid);					// ID
	p16(&p, 5<<11);					// flags
	p16(&p, 1);						// # Zones
	p16(&p, 0);						// # prerequisits
	p16(&p, 2);						// # updates
	p16(&p, 0);						// # additionals

        pname(&p, dnsdomain);		// zone
	p16(&p, Tsoa);					// zone type
	p16(&p, Cin);					// zone class

	/* delete old name */
        pname(&p, dom);				// name
	p16(&p, Ta);					// type
	p16(&p, Call);					// class
	p32(&p, 0);						// TTL
	p16(&p, 0);						// data len

	/* add new A record */
	enc_name(&p, dom);				// name
	p16(&p, Ta);					// type
	p16(&p, Cin);					// class
	p32(&p, 60*60*25);				// TTL (25 hours)
	p16(&p, IPv4addrlen);			// data len
	pmem(&p, addr, IPv4addrlen);	// IP address

	len = p-buf;
	if(write(fd, buf, len) != len)
		sysfatal("write failed: %r");

	notify(ding);
	alarm(3000);
	do{
		if(read(fd, buf, sizeof(buf)) == -1)
			sysfatal("timeout");
		p = buf;
		rxid = g16(&p);
	}while(rxid != txid);
	alarm(0);

	close(fd);

	err = g16(&p) & 7;
	if(err != 0)
		sysfatal("%s", nberr(err));
	exits(0);
}

