/*
 * nbns.c - dump the wins name database from the given server - similar to
 * an axfr request to a DNS server.
 */
#include <u.h>
#include <libc.h>
#include <ip.h>
#include <ctype.h>
#include "netbios.h"

#define TRN		0x8348
#define	RNAMELEN	34

static void
ding(void *u, char *msg)
{
	USED(u);

	if(strstr(msg, "alarm"))
		noted(NCONT);
	noted(NDFLT);
}

static void
usage(void)
{
	fprint(2, "usage: %s [-x netmtpt] server [host]\n", argv0);
	exits("usage");
}

static void
mkrequest(int fd, char *name, int type)
{
	int trn;
	uchar *p, buf[1024];

	p = buf;
	trn = getpid() & 0xffff;

	p16(&p, trn);			// Transaction ID
	p8(&p, 1);			// flags
	p8(&p, 0);			// flags
	p16(&p, 1);			// # questions
	p16(&p, 0);			// # answers
	p16(&p, 0);			// # authorities
	p16(&p, 0);			// # additional
	pnbname(&p, name, type, type);  // question name
	p16(&p, 0x20);			// question type (name query)
	p16(&p, 1);			// question class (inet)

	if(write(fd, buf, p-buf) != p-buf)
		sysfatal("can't write: %r");
}

static void
getreply(int fd)
{
	uchar *p, buf[1024];
	char name[Rnamlen];
	int i, n, trn, ttl, err;

	SET(n);
	trn = getpid() & 0xffff;

	memset(buf, 0, sizeof(buf));
	for(i = 0; i < 4; i++){
		alarm(1000);
		n = read(fd, buf, sizeof(buf));
		alarm(0);

		p = buf;
		if(n != -1 &&  g16(&p) == trn)
			break;
	}
	if(n == -1)
		sysfatal("timeout\n");

	g8(&p);				// flags
	err = g8(&p);			// err
	g16(&p);			// # questions
	g16(&p);			// # answers
	g16(&p);			// # authority answers
	g16(&p);			// # adaitional answers
	gnbname(&p, name, Rnamlen); 	// name in request
 	g16(&p);			// type in request
 	g16(&p);			// class in request
	ttl = g32(&p);			// TTL
	n = g16(&p);			// reply data len

	if(err != 0)
		sysfatal("%s", nberr(err));

	for (i = 0; i < n; i += 6){
		int flgs = g16(&p);
		int a = g8(&p);
		int b = g8(&p);
		int c = g8(&p);
		int d = g8(&p);
		USED(flgs);
		print("sys=%s ip=%d.%d.%d.%d ttl=%d\n", name, a, b, c, d, ttl);
	}
}


void
main(int argc, char **argv)
{
	int fd;
	char net[32];

	setnetmtpt(net, sizeof(net), nil);

	ARGBEGIN{
	case 'x':
		setnetmtpt(net, sizeof(net), EARGF(usage()));
		break;
	}ARGEND;


	if(argc < 1 || argc > 2)
		usage();

	if((fd = dial(netmkaddr(argv[0], "udp", "137"), "137", 0, 0)) < 0)
		sysfatal("%s can't dial - %r", argv[0]);

	notify(ding);

	if(argc == 1){
		mkrequest(fd, "*", '\0');
		getreply(fd);
	}
	else{
		mkrequest(fd, argv[1], ' ');
		getreply(fd);
	}

	close(fd);
}

