#include <u.h>
#include <libc.h>
#include "netbios.h"

static char *errmsgs[] = {
	[0]  "ok",
	[1]  "request format error",
	[2]  "internal server error",
	[3]  "domain name does not exist",
	[4]  "request not supported",
	[5]  "permission denied",
	[6]  "domain name already exists",
	[7]  "resource record already exists",
	[8]  "resource record does not exist",
	[9]  "server not authoritative",
	[10] "domain name not in zone",
};

void
pnbname(uchar **p, char *name, char type, char pad)
{
	char c;
	int i;
	int done = 0;

	*(*p)++ = 0x20;
	for(i = 0; i < 15; i++) {
		if(!done && name[i] == '\0')
			done = 1;
		if(done)
			c = pad;
		else
			c = toupper(name[i]);
		*(*p)++ = (c >> 4) + 'A';
		*(*p)++ = (c & 0xf) + 'A';
	}
	*(*p)++ = (type >> 4) + 'A';
	*(*p)++ = (type & 0xf) + 'A';
	*(*p)++ = 0x0;
}

void
pname(uchar **p, char *s)
{
	uchar *len;

	while (*s){
		len = (*p)++;
		while(*s && *s != '.')
			*(*p)++ = *s++;
		*len = (*p - len) -1;
		if (*s == '.')
			s++;
	}
	*(*p)++ = 0;
}

void
p8(uchar **p, int n)
{
	*(*p)++ = n & 0xff;
}

void
p16(uchar **p, int n)
{
	*(*p)++ = (n >> 8) & 0xff;
	*(*p)++ = n & 0xff;
}

void
p32(uchar **p, int n)
{
	*(*p)++ = (n >> 24) & 0xff;
	*(*p)++ = (n >> 16) & 0xff;
	*(*p)++ = (n >> 8) & 0xff;
	*(*p)++ = n & 0xff;
}

void
pmem(uchar **p, void *v, int len)
{
	char *s;
	for(s = v; len; len--)
		*(*p)++ = *s++;
}

/***********************/

void
gstr(uchar **p, char *s, int len)
{
	char *eos = s+len-1;
	while((*s++ = *(*p)++) && len--)
		continue;
	while(*(*p)++)
		continue;
	*eos = 0;
}

void
gmem(uchar **p, void *v, int len)
{
	char *s;
	for(s = v; len; len--)
		*s++ = *(*p)++;
}

int
g8(uchar **p)
{
	return *(*p)++;
}

int
g16(uchar **p)
{
	int n;

	n = *(*p)++ << 8;
	n |= *(*p)++;
	return n;
}

int
g32(uchar **p)
{
	int n;

	n = *(*p)++ << 24;
	n |= *(*p)++ << 16;
	n |= *(*p)++ << 8;
	n |= *(*p)++;
	return n;
}

int
gnbname(uchar **p, char *s, int len)
{
	int i, type;

	type = *(*p)++;
	for(i = 0; i < len; i++){
		s[i] = *(*p)++ -'A';
		s[i] <<= 4;
		s[i] |= *(*p)++ -'A';
		if(*(*p) == 0){
			(*p)++;
			break;
		}
	}
	s[i] = 0;
	return type;
}

char *
nberr(int n)
{
	static char buf[32];

	if(n < 0 || n >= nelem(errmsgs)){
		snprint(buf, sizeof(buf), "%d unknown error", n);
		return buf;
	}
	return errmsgs[n];
}

